/*
 * $Id: bookmarkform.c,v 1.7 2000/10/03 19:27:25 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "bookmark.h"
#include "bookmarkform.h"
#include "debug.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "util.h"

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static FieldType *fldPtr;

/***********************************************************************
 *
 * FUNCTION:    AddBookmarkFormInit
 *
 * DESCRIPTION: Initialize the add bookmark form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void AddBookmarkFormInit(void)
{
	FormType *addBookmarkForm = FrmGetFormPtr(frmAddBookmark);
    
	fldPtr = GetObjectPtr(frmAddBookmarkName);

	FrmDrawForm(addBookmarkForm);

	FrmSetFocus(addBookmarkForm, FrmGetObjectIndex(addBookmarkForm, frmAddBookmarkName));
}

/***********************************************************************
 *
 * FUNCTION:    AddBookmarkFormHandleEvent
 *
 * DESCRIPTION: Event handler for the add bookmark form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean AddBookmarkFormHandleEvent(const EventType *event)
{
	Boolean handled = false;

	SET_A4_FROM_A5

	switch (event->eType) {
	case frmOpenEvent:
		AddBookmarkFormInit();
		handled = true;
		break;

	case frmCloseEvent:
		handled = false;
		break;

	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmAddBookmarkAdd) {
			if (FldDirty(fldPtr)) {
				Char *name = FldGetTextPtr(fldPtr);
				if (*name)
					AddBookmark(name);
			}
		} else if (event->data.ctlEnter.controlID != frmAddBookmarkCancel) 
			break;

		FrmReturnToForm(Prefs()->toolbar);
		handled = true;
		break;

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    EditBookmarkFormInit
 *
 * DESCRIPTION: Initialize the add bookmark form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void EditBookmarksFormInit(void)
{
	FormType *editBookmarkForm = FrmGetFormPtr(frmEditBookmarks);
	ListType *list = GetObjectPtr(frmEditBookmarksList);

	CreateShortBookmarkList(list);
	LstSetHeight(list, 11);

	FrmDrawForm(editBookmarkForm);
}

/***********************************************************************
 *
 * FUNCTION:    EditBookmarkFormHandleEvent
 *
 * DESCRIPTION: Event handler for the add bookmark form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean EditBookmarksFormHandleEvent(const EventType *event)
{
	Boolean handled = false;

	SET_A4_FROM_A5

	switch (event->eType) {
	case frmOpenEvent:
		EditBookmarksFormInit();
		handled = true;
		break;

	case frmCloseEvent:
		ReleaseBookmarkList();
		handled = false;
		break;

	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmEditBookmarksDelete) {
			ListType *list = GetObjectPtr(frmEditBookmarksList);
			Int16 selection;
			selection = LstGetSelection(list);

			if (selection != noListSelection) {
				Boolean emptyList;

				emptyList = DeleteBookmark(selection);
				if (!emptyList)
					ReleaseBookmarkList();
				CreateShortBookmarkList(list);
				LstSetHeight(list, 11);
				LstDrawList(list);
			}
			break;
		} else if (event->data.ctlEnter.controlID != frmEditBookmarksDone)
			break;

		FrmGotoForm(Prefs()->toolbar);
		handled = true;
		break;

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
