/*
 * $Id: controlform.c,v 1.12 2000/10/22 20:17:32 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "controlform.h"
#include "debug.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "util.h"

static void UpdateControlList(Int16 num) __attribute__ ((section ("plkr")));
static void UpdateAllControlLists(void) __attribute__ ((section ("plkr")));
static void ShowMode(void) __attribute__ ((section ("plkr")));
static void ControlFormInit(void) __attribute__ ((section ("plkr")));
 
/***********************************************************************
 *
 *      Internal structures
 *
 ***********************************************************************/
static SelectType defaultSelect[2][4];
static ModeType defaultMode;

/***********************************************************************
 *
 * FUNCTION:    UpdateControlList
 *
 * DESCRIPTION: Initialize pop-up control list
 *
 * PARAMETERS:  num - list number (starting from zero)
 *
 * RETURN:      Nothing
 *
 * NOTE:        For this function to work the resource IDs must be
 *              located together, i.e. frmControlList2 must be equal
 *              to frmControlList1 + 1, frmControlList3 = frmControlList1 
 *              + 2, and so on.
 *
 ***********************************************************************/
static void UpdateControlList(Int16 num)
{
	ListType *list;
	ControlType *ctl;
	UInt16 selection;

	list = GetObjectPtr(frmControlList1 + num);
	ctl = GetObjectPtr(frmControlPopup1 + num);
	selection = Prefs()->select[Prefs()->controlMode][num];
	LstSetHeight(list, sizeof(Prefs()->select));
	LstSetSelection(list, selection);
	CtlSetLabel(ctl, LstGetSelectionText(list, selection));
}

/***********************************************************************
 *
 * FUNCTION:    UpdateAllControlLists
 *
 * DESCRIPTION: Initialize all pop-up control lists
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void UpdateAllControlLists(void)
{
	Int16 i;

	for (i = 0; i < 4; i++)
		UpdateControlList(i);
}

/***********************************************************************
 *
 * FUNCTION:    ShowMode
 *
 * DESCRIPTION: Show control objects for current mode
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void ShowMode(void)
{
	FormType *controlForm = FrmGetFormPtr(frmControl);

	if (Prefs()->controlMode == MODE1) {
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode2));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode1));

		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup1));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup4));

		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg2));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg3));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg4));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel1));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel4));
	} else if (Prefs()->controlMode == MODE2) {
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode2));

		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup1));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup4));

		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg2));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg3));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg4));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel1));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel4));
	} else {
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, bmpMode3));

		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup2));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup3));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlPopup4));

		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel1));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel2));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel3));
		FrmHideObject(controlForm, FrmGetObjectIndex(controlForm, frmControlLabel4));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg1));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg2));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg3));
		FrmShowObject(controlForm, FrmGetObjectIndex(controlForm, frmControlMsg4));
	}
}

/***********************************************************************
 *
 * FUNCTION:    ControlFormInit
 *
 * DESCRIPTION: Initialize the control options form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void ControlFormInit(void)
{
	FormType *controlForm = FrmGetFormPtr(frmControl);

	MemMove(defaultSelect, Prefs()->select, sizeof(defaultSelect));
	defaultMode = Prefs()->controlMode;

	FrmDrawForm(controlForm);

	CtlSetValue(GetObjectPtr(frmControlMode1), defaultMode == MODE1);
	CtlSetValue(GetObjectPtr(frmControlMode2), defaultMode == MODE2);
	CtlSetValue(GetObjectPtr(frmControlMode3), defaultMode == MODE3);

	ShowMode();
	UpdateAllControlLists();
}

/***********************************************************************
 *
 * FUNCTION:    ControlFormHandleEvent
 *
 * DESCRIPTION: Event handler for the control options form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean ControlFormHandleEvent(const EventType *event)
{
	Boolean handled = false;

	SET_A4_FROM_A5

	switch (event->eType) {
	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmControlMode1) {

			Prefs()->controlMode = MODE1;

			ShowMode();
			UpdateAllControlLists();
			break;

		} else if (event->data.ctlEnter.controlID == frmControlMode2) {

			Prefs()->controlMode = MODE2;

			ShowMode();
			UpdateAllControlLists();
			break;

		} else if (event->data.ctlEnter.controlID == frmControlMode3) {

			Prefs()->controlMode = MODE3;

			ShowMode();
			UpdateAllControlLists();
			break;

		} else if (event->data.ctlEnter.controlID == frmControlCancel) {

			MemMove(Prefs()->select, defaultSelect, sizeof(Prefs()->select));
			Prefs()->controlMode = defaultMode;

		} else if (event->data.ctlEnter.controlID != frmControlOK)
			break;

		FrmReturnToForm(Prefs()->lastForm);
		handled = true;
		break;

	case frmOpenEvent:
		ControlFormInit();
		handled = true;
		break;

	case popSelectEvent:
	{
		UInt16 selection = event->data.popSelect.selection;

		if (selection != noListSelection) {
			ListType *list = event->data.popSelect.listP;
			ControlType *ctl = GetObjectPtr(event->data.popSelect.controlID);
			Char *label = LstGetSelectionText(list, selection);

			CtlSetLabel(ctl, label);
			LstSetSelection(list, selection);

			Prefs()->select[Prefs()->controlMode][event->data.popSelect.listID - frmControlList1] = LstGetSelection(list);
		}
		handled = true;
		break;
	}

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
