/*
 * $Id: db.c,v 1.13 2000/10/03 19:27:25 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "db.h"
#include "debug.h"

/***********************************************************************
 *
 * FUNCTION:    OpenDatabase
 *
 * DESCRIPTION: Open the specified database.
 *
 * PARAMETERS:  db      - upon return, contains the database access 
 *                        pointer, or NULL if the function failed
 *              dbID    - the database ID of the database
 *              cardNo  - card number database resides on
 *              version - upon successful return, the database's 
 *                        specific version number
 *              name    - upon successful return, the database's name
 *              date    - upon successful return, the date the database
 *                        was created
 *
 * RETURN:      The result from the database call or 0 if successful
 *
 ***********************************************************************/
Err OpenDatabase(DmOpenRef *db, const LocalID dbID, const UInt16 cardNo, 
		 UInt16 *version, Char *name, UInt32 *date)
{
	Err err;

	*db = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
	if (!*db) {
		*db = DmOpenDatabase(cardNo, dbID, dmModeReadOnly);
		if (!*db) {
			MSG(_("error code %d\n", DmGetLastErr()));
			return DmGetLastErr();
		}
	}
	err = DmDatabaseInfo(cardNo, dbID, name, NULL, version, date, NULL, 
			     NULL, NULL, NULL, NULL, NULL, NULL);
	MSG_IF(err != 0, "DmDatabaseInfo failed");

	return err;
}

/***********************************************************************
 *
 * FUNCTION:    CloseDatabase
 *
 * DESCRIPTION: Close the specified database
 *
 * PARAMETERS:  db - database access pointer
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void CloseDatabase(DmOpenRef db)
{
	DmCloseDatabase(db);
}
// vim: ts=4
