/*
 * $Id: detailsform.c,v 1.15 2000/10/03 19:27:26 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "debug.h"
#include "detailsform.h"
#include "document.h"
#include "externalform.h"
#include "history.h"
#include "image.h"
#include "link.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "util.h"

/***********************************************************************
 *
 * FUNCTION:    DetailsFormInit
 *
 * DESCRIPTION: Initialize the details form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void DetailsFormInit(void)
{
	FormType *detailsForm = FrmGetFormPtr(frmDetails);
	Int16 reference = GetHistoryDocID();
	FieldType *field = GetObjectPtr(frmDetailsLink);

	AddURLToField(field, reference);
	FrmDrawForm(detailsForm);

	CtlSetValue(GetObjectPtr(frmDetailsStatusRead), 
		    LinkVisited(reference));
	CtlSetValue(GetObjectPtr(frmDetailsStatusUnread), 
		    !LinkVisited(reference));

	CtlSetValue(GetObjectPtr(frmDetailsShowImages), 
		    ShowImages(reference));
}

/***********************************************************************
 *
 * FUNCTION:    DetailsFormHandleEvent
 *
 * DESCRIPTION: Event handler for the details form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean DetailsFormHandleEvent(const EventType *event)
{
	Boolean handled = false;
	Boolean updatePage = false;

	SET_A4_FROM_A5

	switch (event->eType) {
	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmDetailsOK) {
			Int16 reference = GetHistoryDocID();
			Boolean oldStatus = ShowImages(reference);
			Boolean newStatus = CtlGetValue(GetObjectPtr(frmDetailsShowImages));

			updatePage = oldStatus ^ newStatus;

			if (newStatus)
		    		ShowImagesOn(reference);
			else
		    		ShowImagesOff(reference);

			if (CtlGetValue(GetObjectPtr(frmDetailsStatusRead)))
				SetVisitedLink(reference);
			else
				UnsetVisitedLink(reference);

		} else if (event->data.ctlEnter.controlID == frmDetailsCopy) {
			FieldType *field = GetObjectPtr(frmDetailsLink);

			WriteMemo(field);
			handled = true;
			break;
		} else if (event->data.ctlEnter.controlID != frmDetailsCancel)
			break;

		FrmReturnToForm(Prefs()->toolbar);
		if (updatePage) {
			ResetDocumentHeight();
			FrmUpdateForm(Prefs()->toolbar, frmUpdateDoc);
		}
		handled = true;
		break;

	case frmOpenEvent:
		DetailsFormInit();
		handled = true;
		break;

	case frmCloseEvent:
		handled = false;
		break;

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
