/*
 * $Id: externalform.c,v 1.15 2000/10/22 20:17:32 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "const.h"
#include "debug.h"
#include "document.h"
#include "documentdata.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "uncompress.h"
#include "util.h"

void AddURLToField(FieldType *fldPtr, Int16 index) __attribute__ ((section ("plkr")));
void WriteMemo(FieldType *field) __attribute__ ((section ("plkr")));

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static Int16 linkIdx;

/***********************************************************************
 *
 * FUNCTION:    SetLinkIndex
 *
 * DESCRIPTION: Set current link in the ExternalLinks database.
 *
 * PARAMETERS:  index - index of record
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void SetLinkIndex(Int16 index)
{
	linkIdx = index;
}

/***********************************************************************
 *
 * FUNCTION:    AddURLToField
 *
 * DESCRIPTION: Initialize field with URL string
 *
 * PARAMETERS:  fldPtr - pointer to field
 *              index  - index in URL record
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void AddURLToField(FieldType *fldPtr, Int16 index)
{
	MemHandle uncompressHandle = NULL;
	MemHandle linkHandle = NULL;
	Char *text = NULL;
	Document *linkDocument = NULL;
	Int16 count = index;

	if (ReturnDocumentHandle(PLUCKER_LINKS_ID, &linkHandle) == 0) {
		Int16 offset = 0;
		Int16 *numP;

		linkDocument = MemHandleLock(linkHandle);
		numP = (Int16 *) (((void *) linkDocument) + sizeof(Document));

		while(*numP < index) {
			offset = *numP;
			numP += 2;
		}
		numP += 1;
		count -= offset;

		MemHandleUnlock(linkHandle);

		if (ReturnDocumentHandle(*numP, &linkHandle) != 0)
			return;

		linkDocument = (Document *) MemHandleLock(linkHandle);
		if (linkDocument->type == DOCTYPE_LINKS_COMPRESSED) {
			uncompressHandle = Uncompress(linkDocument);
			MemHandleUnlock(linkHandle);
			linkHandle = NULL;
			if (uncompressHandle)
				text = (Char *) MemHandleLock(uncompressHandle);
		}
		else if (linkDocument->type == DOCTYPE_LINKS) {
			text = (Char *)(((void *) linkDocument) + sizeof(Document));
		}

		if (text) {
			while(--count)
				text += StrLen(text) + 1;
			FldInsert(fldPtr, text, StrLen(text));
		}
		if (linkHandle)
			MemHandleUnlock(linkHandle);
		if (uncompressHandle)
			MemHandleUnlock(uncompressHandle);
	}
	if(text == NULL) {
		Char buff[100];
		SysCopyStringResource(buff, strNoURL);
		FldInsert(fldPtr, buff, StrLen(buff));
	}
}

/***********************************************************************
 *
 * FUNCTION:    ExternalLinksFormInit
 *
 * DESCRIPTION: Initialize the externallinks form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void ExternalLinksFormInit(void)
{
	FormType *externallinksForm = FrmGetFormPtr(frmExternalLinks);
	FieldType *field = GetObjectPtr(frmExternalLinksLink);

	AddURLToField(field, linkIdx);
	FrmDrawForm(externallinksForm);
}

/***********************************************************************
 *
 * FUNCTION:    WriteMemo
 *
 * DESCRIPTION: Write the text from a TextField to a Memo.
 *
 * PARAMETERS:  field - field to get text from
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void WriteMemo(FieldType *field)
{
	DmOpenRef MemopadDB;
	MemHandle recH;
	MemPtr recP;
	UInt16 attr;
	UInt16 mode = dmModeReadWrite;
	Int16 category;
	static UInt16 index = dmMaxRecordIndex;

	Char *text = FldGetTextPtr(field);
	UInt16 length = FldGetTextLength(field);

	DateTimeType dateTime;
	Char header[20 + dateStringLength + timeStringLength];
	Char titleStr[] = "Plucker URLs";
	Char dateStr[dateStringLength];
	Char timeStr[timeStringLength];
	Char null ='\0';

	static UInt32 secs = 0;
	static UInt32 offset = 0;

	if(!length)
		return;

	if(!secs) {
		secs = TimGetSeconds();
		TimSecondsToDateTime(secs, &dateTime);
		DateToAscii(dateTime.month, dateTime.day, dateTime.year, 
			    PrefGetPreference(prefDateFormat), (Char *)&dateStr);
		TimeToAscii(dateTime.hour, dateTime.minute, 
				PrefGetPreference(prefTimeFormat), (Char *)&timeStr);
		StrPrintF(header, "%s %s %s\n", titleStr, dateStr, timeStr);
	} else {
		StrCopy(header, "\n");
	}

	if((MemopadDB = DmOpenDatabaseByTypeCreator('DATA', 'memo', mode)) == NULL)
		return;
	 
	if(index == dmMaxRecordIndex)
		recH = DmNewRecord(MemopadDB, &index, StrLen(header) + length + 1);
	else
		recH = DmResizeRecord(MemopadDB, index, offset + StrLen(header) + length + 1);

	if(recH != NULL) { // Should we pop up an alert form here??
		recP = MemHandleLock(recH);

		DmWrite(recP, offset, header, StrLen(header));
		DmWrite(recP, offset + StrLen(header), text, length);
		DmWrite(recP, offset + StrLen(header) + length, &null, 1);
	  
		MemHandleUnlock(recH);

		DmRecordInfo(MemopadDB, index, &attr, NULL, NULL);
		attr &= ~dmRecAttrCategoryMask;

		category = CategoryFind(MemopadDB, "Plucker");

		if (category == dmAllCategories)
			attr |= dmUnfiledCategory;
		else
			attr |= category;

		DmSetRecordInfo(MemopadDB, index, &attr, NULL);
		DmReleaseRecord(MemopadDB, index, true);

		offset += StrLen(header) + length;
	}
	DmCloseDatabase(MemopadDB);
}

/***********************************************************************
 *
 * FUNCTION:    ExternalLinksFormHandleEvent
 *
 * DESCRIPTION: Event handler for the externallinks form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean ExternalLinksFormHandleEvent(const EventType *event)
{
	Boolean handled = false;
	FieldType *field = GetObjectPtr(frmExternalLinksLink);

	SET_A4_FROM_A5

	switch (event->eType) {
	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmExternalLinksCopy)
			WriteMemo(field);
		else if (event->data.ctlEnter.controlID != frmExternalLinksBack)
			break;

		FrmGotoForm(Prefs()->toolbar);
		handled = true;
		break;

	case frmOpenEvent:
		ExternalLinksFormInit();
		handled = true;
		break;

	case frmCloseEvent:
		handled = false;
		break;

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
