/*
 * $Id: link.c,v 1.17 2000/10/03 19:27:30 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "const.h"
#include "debug.h"
#include "documentdata.h"
#include "link.h"

/***********************************************************************
 *
 * FUNCTION:    LinkVisited
 *
 * DESCRIPTION: Get the status of the link.
 *
 * PARAMETERS:  reference - the record number the link refers to
 *
 * RETURN:      The status of the link (visited = true)
 *
 ***********************************************************************/
Boolean LinkVisited(Int16 reference)
{
	return GetBitStatus(LINK_TABLE_ID, reference);
}

/***********************************************************************
 *
 * FUNCTION:    SetVisitedLink
 *
 * DESCRIPTION: Set the status to visited for the link.
 *
 * PARAMETERS:  reference - the record number the link refers to
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void SetVisitedLink(Int16 reference)
{
	SetBitStatus(LINK_TABLE_ID, reference, true);
}

/***********************************************************************
 *
 * FUNCTION:    UnsetVisitedLink
 *
 * DESCRIPTION: Set the status to not visited for the link.
 *
 * PARAMETERS:  reference - the record number the link refers to
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void UnsetVisitedLink(Int16 reference)
{
	SetBitStatus(LINK_TABLE_ID, reference, false);
}
// vim: ts=4
