/*
 * $Id: prefsform.c,v 1.20 2000/10/22 20:17:33 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "const.h"
#include "debug.h"
#include "document.h"
#include "documentdata.h"
#include "prefsdata.h"
#include "prefsform.h"
#include "resourceids.h"
#include "screen.h"
#include "util.h"

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static ScrollbarType scrollbar;
static ScreenModeType screenDepth;
static UInt16 toolbarButton;

/***********************************************************************
 *
 * FUNCTION:    PrefsFormInit
 *
 * DESCRIPTION: Initialize the preference form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void PrefsFormInit(void)
{
	ListType *list;
	ControlType *ctl;
	UInt16 selection;
	FormType *prefsForm = FrmGetFormPtr(frmPref);

	FrmDrawForm(prefsForm);

	CtlSetValue(GetObjectPtr(frmPrefStrikethrough), Prefs()->strikethrough);
	CtlSetValue(GetObjectPtr(frmPrefUnderline), Prefs()->underlineMode);
	CtlSetValue(GetObjectPtr(frmPrefLinkClick), Prefs()->linkClick);

	list = GetObjectPtr(frmPrefScrollbarList);
	ctl = GetObjectPtr(frmPrefScrollbarPopup);
	selection = scrollbar = Prefs()->scrollbar;

	LstSetSelection(list, selection);
	CtlSetLabel(ctl, LstGetSelectionText(list, selection));

	list = GetObjectPtr(frmPrefScreenDepthList);
	ctl = GetObjectPtr(frmPrefScreenDepthPopup);
	screenDepth = Prefs()->screenDepth;

	switch (screenDepth) {
	case BLACKWHITE:
		selection = 0;
		break;
	case GREYSCALE4:
		selection = 1;
		break;
	case GREYSCALE16:
		selection = 2;
		break;
	case COLOR256:
		selection = 3;
		break;
	default:
		selection = 0;
	}
	LstSetSelection(list, selection);
	CtlSetLabel(ctl, LstGetSelectionText(list, selection));

	list = GetObjectPtr(frmPrefMenuToolbarList);
	ctl = GetObjectPtr(frmPrefMenuToolbarPopup);
	toolbarButton = Prefs()->toolbarButton;
	LstSetSelection(list, toolbarButton);
	CtlSetLabel(ctl, LstGetSelectionText(list, toolbarButton));
}

/***********************************************************************
 *
 * FUNCTION:    PrefsFormHandleEvent
 *
 * DESCRIPTION: Event handler for the preference form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if event was handled
 *
 ***********************************************************************/
Boolean PrefsFormHandleEvent(const EventType *event)
{
	Boolean handled = false;
	Boolean updateDoc = false;

	SET_A4_FROM_A5

	switch (event->eType) {
	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmPrefOK) {
			updateDoc = true;

			Prefs()->strikethrough = CtlGetValue(GetObjectPtr(frmPrefStrikethrough));
			Prefs()->underlineMode = CtlGetValue(GetObjectPtr(frmPrefUnderline));
			Prefs()->linkClick = CtlGetValue(GetObjectPtr(frmPrefLinkClick));
			Prefs()->scrollbar = scrollbar;
			Prefs()->toolbarButton = toolbarButton;

			if (screenDepth != Prefs()->screenDepth) {
				Prefs()->screenDepth = screenDepth;
				SetScreenMode();
			}
		} else if (event->data.ctlEnter.controlID != frmPrefCancel)
			break;

		FrmReturnToForm(Prefs()->lastForm);
		if (updateDoc)
			FrmUpdateForm(Prefs()->lastForm, frmUpdateDoc);
		handled = true;
		break;

	case frmOpenEvent:
		PrefsFormInit();
		handled = true;
		break;

        case popSelectEvent:
        {
		if (event->data.popSelect.controlID == frmPrefScrollbarPopup) {
                        ListType *list = event->data.popSelect.listP;
                	UInt16 selection = event->data.popSelect.selection;

                	if (selection != noListSelection) {
                        	ControlType *ctl = GetObjectPtr(frmPrefScrollbarPopup);
                        	Char *label = LstGetSelectionText(list, selection);

                        	CtlSetLabel(ctl, label);
                        	LstSetSelection(list, selection);

                        	scrollbar = LstGetSelection(list);
			}
                	handled = true;
                } else if (event->data.popSelect.controlID == frmPrefScreenDepthPopup) {
                        ListType *list = event->data.popSelect.listP;
                	UInt16 selection = event->data.popSelect.selection;

                	if (selection != noListSelection) {
                        	ControlType *ctl = GetObjectPtr(frmPrefScreenDepthPopup);
                        	Char *label = LstGetSelectionText(list, selection);

                        	CtlSetLabel(ctl, label);
                        	LstSetSelection(list, selection);

                        	screenDepth = (ScreenModeType) StrAToI(label);
			}
                	handled = true;
                } else if (event->data.popSelect.controlID == frmPrefMenuToolbarPopup) {
                        ListType *list = event->data.popSelect.listP;
                	UInt16 selection = event->data.popSelect.selection;

                	if (selection != noListSelection) {
                        	ControlType *ctl = GetObjectPtr(frmPrefMenuToolbarPopup);
                        	Char *label = LstGetSelectionText(list, selection);

                        	CtlSetLabel(ctl, label);
                        	LstSetSelection(list, selection);

                        	toolbarButton = selection;
			}
                	handled = true;
                }
                break;
        }

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
