/*
 * $Id: screen.c,v 1.13 2000/10/03 19:27:34 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "debug.h"
#include "mainform.h"
#include "os.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "screen.h"

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static UInt32 screenDepth = 1;

/***********************************************************************
 *
 * FUNCTION:    SetScreenModeOS3
 *
 * DESCRIPTION: Set screen bit depth for PalmOS3
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 * NOTE:        Only works with PalmOS 3.x
 *
 *              If the function fails to set the requested
 *              screen depth (from preference) it will fall 
 *              back to the default screen depth for the device
 *
 ***********************************************************************/
void SetScreenModeOS3(void)
{ 
	Err err;
	UInt16 activeFormID;
	
	if (screenDepth == Prefs()->screenDepth)
		return;

	screenDepth = Prefs()->screenDepth;

	err = WinScreenMode(winScreenModeSet, NULL, NULL, &screenDepth, NULL);
	if (err) {
		WinScreenMode(winScreenModeGetDefaults, NULL, NULL, &screenDepth, NULL);
		Prefs()->screenDepth = screenDepth;
		WinScreenMode(winScreenModeSetToDefaults, NULL, NULL, NULL, NULL);
	}
	activeFormID = FrmGetActiveFormID();
	if (activeFormID == frmMainTop || activeFormID == frmMainBottom) {
		FrmEraseForm(FrmGetFormPtr(Prefs()->toolbar));
		MainFormInit();
	}
}

/***********************************************************************
 *
 * FUNCTION:    SetScreenModeOS2
 *
 * DESCRIPTION: Set screen bit depth for PalmOS2
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void SetScreenModeOS2(void)
{ 
	screenDepth = Prefs()->screenDepth = BLACKWHITE;
}

/***********************************************************************
 *
 * FUNCTION:    RestoreScreenModeOS3
 *
 * DESCRIPTION: Restore screen mode (PalmOS 3.x)
 *
 * PARAMETERS:  None
 *
 * RETURN:      True if successful
 *
 ***********************************************************************/
Boolean RestoreScreenModeOS3(void)
{
	Err err = WinScreenMode(winScreenModeSetToDefaults, NULL, NULL, NULL, NULL);
	return !err;
}

/***********************************************************************
 *
 * FUNCTION:    RestoreScreenModeOS2
 *
 * DESCRIPTION: Restore screen mode (PalmOS2)
 *
 * PARAMETERS:  None
 *
 * RETURN:      True if successful
 *
 ***********************************************************************/
Boolean RestoreScreenModeOS2(void)
{
	return true;
}

/***********************************************************************
 *
 * FUNCTION:    TooHighBitDepth_35
 *
 * DESCRIPTION: Check if the current device can handle the bit 
 *              depth of the image (OS 3.5 version)
 *
 * PARAMETERS:  depth - bit depth
 *
 * RETURN:      Always false
 *
 * NOTE:        Instead of calling this function directly you should
 *              use the macro TooHighBitDepth(depth) which will use
 *              the correct function depending on the device's OS
 *
 ***********************************************************************/
Boolean TooHighBitDepth_35(UInt32 depth)
{
	return false;
}

/***********************************************************************
 *
 * FUNCTION:    TooHighBitDepth_Default
 *
 * DESCRIPTION: Check if the current device can handle the bit 
 *              depth of the image (default version)
 *
 * PARAMETERS:  depth - bit depth
 *
 * RETURN:      True if the device can't handle the image's bit depth
 *
 * NOTE:        Instead of calling this function directly you should
 *              use the macro TooHighBitDepth(depth) which will use
 *              the correct function depending on the device's OS
 *
 ***********************************************************************/
Boolean TooHighBitDepth_Default(UInt32 depth)
{
	return (depth > screenDepth);
}
// vim: ts=4
