/*
 * $Id: searchform.c,v 1.15 2000/10/03 19:27:34 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "debug.h"
#include "document.h"
#include "prefsdata.h"
#include "resourceids.h"
#include "search.h"
#include "searchform.h"
#include "util.h"

/***********************************************************************
 *
 *      Internal Constants
 *
 ***********************************************************************/
#define START_AT_TOP true

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static FieldType *fldPtr;
static Boolean searchInAllPages;

/***********************************************************************
 *
 * FUNCTION:    SearchFormInit
 *
 * DESCRIPTION: Initialize the search form.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void SearchFormInit(void)
{
	Char text[16];
	ListType *list;
        ControlType *ctl;
        UInt16 selection;
	FormType *searchForm = FrmGetFormPtr(frmSearch);
    
	fldPtr = GetObjectPtr(frmSearchField);
	FldSetSelection(fldPtr, 0, 0);
	GetSearchString(text);
	if (*text) {
		FldInsert(fldPtr, text, StrLen(text));
		FldSetSelection(fldPtr, 0, StrLen(text));
	}
	FrmDrawForm(searchForm);
	FrmSetFocus(searchForm, FrmGetObjectIndex(searchForm, frmSearchField));

	searchInAllPages = Prefs()->searchInAllPages;
	if (searchInAllPages)
		selection = 1;
	else
		selection = 0;

	list = GetObjectPtr(frmSearchList);
        ctl = GetObjectPtr(frmSearchPopup);
	LstSetSelection(list, selection);
	CtlSetLabel(ctl, LstGetSelectionText(list, selection));

	CtlSetValue(GetObjectPtr(frmSearchCaseless), Prefs()->caseSensitive);
}

/***********************************************************************
 *
 * FUNCTION:    SearchFormHandleEvent
 *
 * DESCRIPTION: Event handler for the search form.
 *
 * PARAMETERS:  event - pointer to an EventType structure
 *
 * RETURN:      True if the event was handled
 *
 ***********************************************************************/
Boolean SearchFormHandleEvent(const EventType *event)
{
	Boolean handled = false;
	Boolean found = false;
	Char *pattern = NULL;

	SET_A4_FROM_A5

	switch (event->eType) {
	case ctlSelectEvent:
		if (event->data.ctlEnter.controlID == frmSearchOK) {
			Prefs()->caseSensitive = CtlGetValue(GetObjectPtr(frmSearchCaseless));
			Prefs()->searchInAllPages = searchInAllPages;
			pattern = FldGetTextPtr(fldPtr);
			if (pattern) {
				SetSearchString(pattern);

				if (!searchInAllPages) {
					found = SearchInPage(pattern, START_AT_TOP);
				} else {
					FrmGotoForm(frmResult);
					handled = true;
					break;
				}
			}
		} else if (event->data.ctlEnter.controlID == frmSearchHistory) {
			Int16 selection;
			Int16 entries = Prefs()->searchEntries;
			ListType *list = GetObjectPtr(frmSearchHistoryList);
			MemHandle nameListHandle;
			Char **nameList;
			Char *patternList;

			if (entries == 0) {
				FrmAlert(infoEmptyPatterns);
				break;
			}

			patternList = GetSearchHistory();
			nameListHandle = SysFormPointerArrayToStrings(patternList,entries);
			nameList = MemHandleLock(nameListHandle);
			LstSetListChoices(list, nameList, entries);
			LstSetHeight(list, entries);
			selection = LstPopupList(list);
			if (selection != noListSelection) {
				Char *pattern;
				Char *label = LstGetSelectionText(list, selection);

				pattern = FldGetTextPtr(fldPtr);
				if (pattern)
					FldSetSelection(fldPtr, 0, StrLen(pattern));
				FldInsert(fldPtr, label, StrLen(label));
				FldSetSelection(fldPtr, 0, StrLen(label));
			}
			MemHandleUnlock(nameListHandle);
			MemHandleFree(nameListHandle);
			break;
		} else if (event->data.ctlEnter.controlID != frmSearchCancel) 
			break;

		FrmReturnToForm(Prefs()->toolbar);
		if (found)
			FrmUpdateForm(Prefs()->toolbar, frmShowResult);
		handled = true;
		break;

	case frmCloseEvent:
		handled = false;
		break;

	case frmOpenEvent:
		SearchFormInit();
		handled = true;
		break;

	case popSelectEvent:
		if (event->data.popSelect.controlID == frmSearchPopup) {
			ListType *list = event->data.popSelect.listP;
			UInt16 selection = event->data.popSelect.selection;

			if (selection != noListSelection) {
				ControlType *ctl = GetObjectPtr(frmSearchPopup);
				Char *label = LstGetSelectionText (list, selection);

				CtlSetLabel (ctl, label);
				LstSetSelection (list, selection);
				searchInAllPages = (selection == 1);
			}
			handled = true;
		}
		break;

	default:
		handled = false;
	}

	RESTORE_A4

	return handled;
}
// vim: ts=4
