/* 
 * $Id: session.c,v 1.20 2000/10/03 19:27:34 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "const.h"
#include "debug.h"
#include "documentdata.h"
#include "history.h"
#include "session.h"

/***********************************************************************
 *
 * FUNCTION:    HandleSessionData
 *
 * DESCRIPTION: Handle the session data.
 *
 * PARAMETERS:  init - if true initialize session data, otherwise
 *                     store session data
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
static void HandleSessionData(Boolean init)
{
	MemHandle handle = NULL;
	void *sessionPtr;

	if (ReturnDynamicHandle(SESSION_DATA_ID, 0, &handle) != 0) {
		if (init)
			InitHistory();
		return;
	}
	sessionPtr = MemHandleLock(handle);
	FATAL_IF(sessionPtr == NULL, "MemHandleLock failed\n");
	ErrFatalDisplayIf(sessionPtr == NULL, "InitSessionData: MemHandleLock failed");

	if (init)
		ReadHistory(sessionPtr);
	else
		SaveHistory(sessionPtr);

	MemHandleUnlock(handle);
}

/***********************************************************************
 *
 * FUNCTION:    InitSessionData
 *
 * DESCRIPTION: Initialize the session data.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void InitSessionData(void)
{
	HandleSessionData(true);
}

/***********************************************************************
 *
 * FUNCTION:    SaveSessionData
 *
 * DESCRIPTION: Save the session data.
 *
 * PARAMETERS:  None
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void SaveSessionData(void)
{
	HandleSessionData(false);
}
// vim: ts=4
