/*
 * $Id: util.c,v 1.14 2000/10/03 19:27:35 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (C) 1998, Mark Ian Lillywhite and Michael Nordstrm
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Mark Lillywhite - plucker@rubberchicken.org
 */

#include "const.h"
#include "debug.h"
#include "prefsdata.h"
#include "util.h"

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: Return a pointer to an object in a form.
 *
 * PARAMETERS:  objectID - ID of an object in the form
 *
 * RETURN:      Pointer to an object in the form
 *
 ***********************************************************************/
void *GetObjectPtr(const Int16 objectID)
{
	FormType *frm = FrmGetActiveForm();

	return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID));
}

/***********************************************************************
 *
 * FUNCTION:    TrimText
 *
 * DESCRIPTION: Trim text for specified length
 *
 * PARAMETERS:  text - pointer to text
 *              len  - max lenght of string, i.e. trim up to this length
 *
 * RETURN:      Nothing
 *
 ***********************************************************************/
void TrimText(Char *text, Int16 len)
{
	if (FntCharsWidth(text, StrLen(text)) > len) {
		while (FntCharsWidth(text, StrLen(text)) > (len - 6))
			text[StrLen(text)-1] = 0;
		StrCat(text, "\205");
	}
}

/***********************************************************************
 *
 * FUNCTION:    TopLeftX
 *
 * DESCRIPTION: Return X coordinate for viewport boundary
 *
 * PARAMETERS:  None
 *
 * RETURN:      X coordinate
 *
 ***********************************************************************/
Int16 TopLeftX(void)
{
	return (Prefs()->scrollbar == SCROLLBAR_LEFT ? TOP_LEFT_X + 7 : TOP_LEFT_X);
}

/***********************************************************************
 *
 * FUNCTION:    TopLeftY
 *
 * DESCRIPTION: Return Y coordinate for viewport boundary
 *
 * PARAMETERS:  None
 *
 * RETURN:      Y coordinate
 *
 ***********************************************************************/
Int16 TopLeftY(void)
{
	if (Prefs()->toolbar == TOOLBAR_TOP)
		return TOP_LEFT_Y;
	else
		return 0;
}

/***********************************************************************
 *
 * FUNCTION:    ExtentY
 *
 * DESCRIPTION: Return height for viewport boundary
 *
 * PARAMETERS:  None
 *
 * RETURN:      height
 *
 ***********************************************************************/
Int16 ExtentY(void)
{
	if (Prefs()->toolbar == TOOLBAR_NONE)
		return EXTENT_Y + 15;
	else
		return EXTENT_Y;
}
// vim: ts=4
